#include <linux/version.h>

#include <linux/string.h>
#include <linux/errno.h>
#include <linux/module.h>

#include "nikal.h"

#ifdef nNIKAL100_kCopyrightString
   MODULE_LICENSE(nNIKAL100_kCopyrightString);
#endif

#ifdef nNIKAL100_kAuthorString
   MODULE_AUTHOR(nNIKAL100_kAuthorString);
#endif

#ifdef nNIKAL100_kDescriptionString
   MODULE_DESCRIPTION(nNIKAL100_kDescriptionString);
#endif


#include <linux/acpi.h>
#include <linux/mod_devicetable.h>

int nNIAPAL200_driverEntry(struct nNIKAL100_tDriver *driver);

static struct nNIKAL100_tDriver atomicDriver;
static void* nativeDriverRegistration;

/* native driver registration functions. These functions are auto-generated */
void *registerNativeDriver(nNIKAL100_tDriver*);
void unregisterNativeDriver(void*);

int init_module(void)
{
   atomicDriver.module = THIS_MODULE;

   if (nNIAPAL200_driverEntry(&atomicDriver))
      return -ENODEV;

   nativeDriverRegistration = registerNativeDriver(&atomicDriver);

   if (nativeDriverRegistration == NULL)
   {
      if (atomicDriver.unload)
         atomicDriver.unload(&atomicDriver);
      return -ENODEV;
   }

   return 0;
}

void cleanup_module(void)
{
   unregisterNativeDriver(nativeDriverRegistration);

   if (atomicDriver.unload)
      atomicDriver.unload(&atomicDriver);
}

static const struct acpi_device_id niacpiidtable[] = {
   {"NIC77C3", (unsigned long)&atomicDriver},
   {"NIC79D6", (unsigned long)&atomicDriver},
   {"NIC7019", (unsigned long)&atomicDriver},
    { },
};
MODULE_DEVICE_TABLE(acpi, niacpiidtable);

static struct acpi_driver niAcpiDriver = {
   .name = "nicntdrk",
   .class= "National Instruments",
   .ids  = niacpiidtable,
};

static int acpiProbe(void *dev)
{
   return nNIKAL1500_helperACPIProbe(dev, &atomicDriver);
}

void *registerNativeDriver(nNIKAL100_tDriver* ignore)
{
   return nNIKAL1500_registerACPIDriver(&niAcpiDriver, &acpiProbe);
}

void unregisterNativeDriver(void* data)
{
   nNIKAL1500_unregisterACPIDriver(data);
}


#define nNIKAL100_mExportSymbol EXPORT_SYMBOL
